//
//  AnimationLayer.m
//  ColorsWar
//
/*
 * HAZ™ Dev Group: http://imhaz.com
 *
 * Copyright (c) 2007-2011 HAZ™ Inc.
 *
 * Written by Hercy Chang 2011-12-08
 *
 */

#import "AnimationLayer.h"
#import "GameView.h"
#import "DataLayer.h"
#import "CWColor.h"

@implementation AnimationLayer

static AnimationLayer* instanceOfAML;
+(AnimationLayer*) sharedAnimationLayer
{
	NSAssert(instanceOfAML != nil, @"GameScene instance not yet initialized!");
	return instanceOfAML;
}

void drawExplosionWave( CGPoint center,float r1, float r2, CGRect color1, CGRect color2, int numofpoint)
{
	const float coef = (float)M_PI*2/numofpoint;
	
	GLfloat *vertices = calloc( sizeof(GLfloat)*2*(numofpoint+1)*2, 1);
	GLfloat *colors = calloc( sizeof(GLfloat)*4*(numofpoint+1)*2, 1);
	if( ! vertices || ! colors)
		return;
	
	vertices[0] = center.x;
	vertices[1] = center.y+r1;
	vertices[2] = center.x;
	vertices[3] = center.y+r2;
	colors[0] = color1.origin.x;
	colors[1] = color1.origin.y;
	colors[2] = color1.size.width;
	colors[3] = color1.size.height;
	colors[4] = color2.origin.x;
	colors[5] = color2.origin.y;
	colors[6] = color2.size.width;
	colors[7] = color2.size.height;
	
	for(NSUInteger i=1;i<(numofpoint+1);i++)
	{
		float rads = i*coef;
		GLfloat ix1 = r1 * sinf(rads) + center.x;
		GLfloat iy1 = r1 * cosf(rads) + center.y;
		GLfloat ix2 = r2 * sinf(rads) + center.x;
		GLfloat iy2 = r2 * cosf(rads) + center.y;
		
		vertices[i*4] = ix1 * CC_CONTENT_SCALE_FACTOR();
		vertices[i*4+1] = iy1 * CC_CONTENT_SCALE_FACTOR();
		vertices[i*4+2] = ix2 * CC_CONTENT_SCALE_FACTOR();
		vertices[i*4+3] = iy2 * CC_CONTENT_SCALE_FACTOR();
		
		colors[i*8] = color1.origin.x;
		colors[i*8+1] = color1.origin.y;
		colors[i*8+2] = color1.size.width;
		colors[i*8+3] = color1.size.height;
		
		colors[i*8+4] = color2.origin.x;
		colors[i*8+5] = color2.origin.y;
		colors[i*8+6] = color2.size.width;
		colors[i*8+7] = color2.size.height;
	}
	
	// Default GL states: GL_TEXTURE_2D, GL_VERTEX_ARRAY, GL_COLOR_ARRAY, GL_TEXTURE_COORD_ARRAY
	// Needed states: GL_VERTEX_ARRAY, 
	// Unneeded states: GL_TEXTURE_2D, GL_TEXTURE_COORD_ARRAY, GL_COLOR_ARRAY	
	glDisable(GL_TEXTURE_2D);
	glDisableClientState(GL_TEXTURE_COORD_ARRAY);
	//glDisableClientState(GL_COLOR_ARRAY);
	
	glColorPointer(4, GL_FLOAT, 0, colors);
	glVertexPointer(2, GL_FLOAT, 0, vertices);	
	glDrawArrays(GL_TRIANGLE_STRIP, 0, (numofpoint+1)*2);
	
	// restore default state
	glEnableClientState(GL_COLOR_ARRAY);
	glEnableClientState(GL_TEXTURE_COORD_ARRAY);
	glEnable(GL_TEXTURE_2D);	
	
	free( vertices );
	free( colors );
	
}

void drawBullet( CGPoint center, float r,float rota,CGRect color1,CGRect color2,int numofpoint )
{
	const float coef = (float)M_PI*2/numofpoint;
	float additionangle = rota * (float)M_PI/180;
	
	GLfloat *vertices = calloc( sizeof(GLfloat)*2*(numofpoint+2), 1);
	GLfloat *colors = calloc( sizeof(GLfloat)*4*(numofpoint+2), 1);
	if( ! vertices || ! colors)
		return;
	
	vertices[0] = center.x;
	vertices[1] = center.y;
	colors[0] = color1.origin.x;
	colors[1] = color1.origin.y;
	colors[2] = color1.size.width;
	colors[3] = color1.size.height;
	
	for(NSUInteger i=1;i<(numofpoint+2);i++)
	{
		float rads = (i-1)*coef + additionangle;
		GLfloat ix = r * sinf(rads) + center.x;
		GLfloat iy = r * cosf(rads) + center.y;
		
		vertices[i*2] = ix * CC_CONTENT_SCALE_FACTOR();
		vertices[i*2+1] = iy * CC_CONTENT_SCALE_FACTOR();
		
		colors[i*4] = color2.origin.x;
		colors[i*4+1] = color2.origin.y;
		colors[i*4+2] = color2.size.width;
		colors[i*4+3] = color2.size.height;
	}
	
	// Default GL states: GL_TEXTURE_2D, GL_VERTEX_ARRAY, GL_COLOR_ARRAY, GL_TEXTURE_COORD_ARRAY
	// Needed states: GL_VERTEX_ARRAY, 
	// Unneeded states: GL_TEXTURE_2D, GL_TEXTURE_COORD_ARRAY, GL_COLOR_ARRAY	
	glDisable(GL_TEXTURE_2D);
	glDisableClientState(GL_TEXTURE_COORD_ARRAY);
	//glDisableClientState(GL_COLOR_ARRAY);
	
	glColorPointer(4, GL_FLOAT, 0, colors);
	glVertexPointer(2, GL_FLOAT, 0, vertices);	
	glDrawArrays(GL_TRIANGLE_FAN, 0, numofpoint+2);
	
	// restore default state
	glEnableClientState(GL_COLOR_ARRAY);
	glEnableClientState(GL_TEXTURE_COORD_ARRAY);
	glEnable(GL_TEXTURE_2D);	
	
	free( vertices );
	free( colors );
}

void drawLaser(CGPoint sp,CGPoint ep,CGRect startc,CGRect endc,CGRect edgecolor,float width)
{
	float anglestoe = AngleOfTwoPoints(sp, ep);
	float radfactor = (float)M_PI/180;
	
	GLfloat vertices[12];
	GLfloat colors[24];
	
	float dx1 = width/2*sinf((anglestoe+90)*radfactor);
	float dy1 = width/2*cosf((anglestoe+90)*radfactor);
	float dx2 = width/2*sinf((anglestoe+270)*radfactor);
	float dy2 = width/2*cosf((anglestoe+270)*radfactor);
	
	vertices[0] = sp.x+dx1;
	vertices[1] = sp.y+dy1;
	vertices[2] = ep.x+dx1;
	vertices[3] = ep.y+dy1;
	vertices[4] = sp.x;
	vertices[5] = sp.y;
	vertices[6] = ep.x;
	vertices[7] = ep.y;
	vertices[8] = sp.x+dx2;
	vertices[9] = sp.y+dy2;
	vertices[10] = ep.x+dx2;
	vertices[11] = ep.y+dy2;
	
	colors[0] = colors[4] = colors[16] = colors[20] = edgecolor.origin.x;
	colors[1] = colors[5] = colors[17] = colors[21] = edgecolor.origin.y;
	colors[2] = colors[6] = colors[18] = colors[22] = edgecolor.size.width;
	colors[3] = colors[7] = colors[19] = colors[23] = edgecolor.size.height;
	
	colors[8] = startc.origin.x;
	colors[9] = startc.origin.y;
	colors[10] = startc.size.width;
	colors[11] = startc.size.height;
	
	colors[12] = endc.origin.x;
	colors[13] = endc.origin.y;
	colors[14] = endc.size.width;
	colors[15] = endc.size.height;
	
	glDisable(GL_TEXTURE_2D);
	glDisableClientState(GL_TEXTURE_COORD_ARRAY);
	
	glColorPointer(4, GL_FLOAT, 0, colors);
	glVertexPointer(2, GL_FLOAT, 0, vertices);	
	glDrawArrays(GL_TRIANGLE_STRIP, 0, 6);
	
	// restore default state
	glEnableClientState(GL_TEXTURE_COORD_ARRAY);
	glEnable(GL_TEXTURE_2D);		
	
}

void advancedDEffect( CGPoint center,float r1, float r2, CGRect sc1,CGRect ec1 ,CGRect sc2,CGRect ec2, int numofpoint,float radstart,float radend)
{
	if(r1==r2)return;
	
	float coef = (radend-radstart)/numofpoint;
	float pif = (float)M_PI/180;
	
	GLfloat *vertices = calloc( sizeof(GLfloat)*2*(numofpoint+1)*2, 1);
	GLfloat *colors = calloc( sizeof(GLfloat)*4*(numofpoint+1)*2, 1);
	if( ! vertices || ! colors)
		return;
	
	for(NSUInteger i=0;i<(numofpoint+1);i++)
	{
		float rads = (i*coef + radstart)*pif;
		GLfloat ix1 = r1 * sinf(rads) + center.x;
		GLfloat iy1 = r1 * cosf(rads) + center.y;
		GLfloat ix2 = r2 * sinf(rads) + center.x;
		GLfloat iy2 = r2 * cosf(rads) + center.y;
		
		vertices[i*4] = ix1 * CC_CONTENT_SCALE_FACTOR();
		vertices[i*4+1] = iy1 * CC_CONTENT_SCALE_FACTOR();
		vertices[i*4+2] = ix2 * CC_CONTENT_SCALE_FACTOR();
		vertices[i*4+3] = iy2 * CC_CONTENT_SCALE_FACTOR();
		
		float trf = sinf((float)i/numofpoint*3.1415926);
		CGRect color1 = CWColorTransform(sc1, ec1, trf);
		CGRect color2 = CWColorTransform(sc2, ec2, trf);
		
		colors[i*8] = color1.origin.x;
		colors[i*8+1] = color1.origin.y;
		colors[i*8+2] = color1.size.width;
		colors[i*8+3] = color1.size.height;
		
		colors[i*8+4] = color2.origin.x;
		colors[i*8+5] = color2.origin.y;
		colors[i*8+6] = color2.size.width;
		colors[i*8+7] = color2.size.height;
	}
	
	// Default GL states: GL_TEXTURE_2D, GL_VERTEX_ARRAY, GL_COLOR_ARRAY, GL_TEXTURE_COORD_ARRAY
	// Needed states: GL_VERTEX_ARRAY, 
	// Unneeded states: GL_TEXTURE_2D, GL_TEXTURE_COORD_ARRAY, GL_COLOR_ARRAY	
	glDisable(GL_TEXTURE_2D);
	glDisableClientState(GL_TEXTURE_COORD_ARRAY);
	//glDisableClientState(GL_COLOR_ARRAY);
	
	glColorPointer(4, GL_FLOAT, 0, colors);
	glVertexPointer(2, GL_FLOAT, 0, vertices);	
	glDrawArrays(GL_TRIANGLE_STRIP, 0, (numofpoint+1)*2);
	
	// restore default state
	glEnableClientState(GL_COLOR_ARRAY);
	glEnableClientState(GL_TEXTURE_COORD_ARRAY);
	glEnable(GL_TEXTURE_2D);	
	
	free( vertices );
	free( colors );
	
}

-(void)draw{
	if([CCDirector sharedDirector].isPaused)
		return;
	GameView *gview = [GameView sharedGameView];
	//draw the bullets
	for(NSMutableDictionary *abullet in gview.mc.bullets){
		int sh = [[abullet objectForKey:kBShape] intValue];			
		CGPoint bulletsp = [[abullet objectForKey:kBStart] CGPointValue];
		CGPoint bulletpos = [[abullet objectForKey:kBPosition] CGPointValue];
		CGRect bulletcolor = [[abullet objectForKey:kBColor] CGRectValue];
		float bulletrota = [[abullet objectForKey:kBRotation] floatValue];
		float bulletrange = [[abullet objectForKey:kBRange] floatValue];
		float bulletradius = [[abullet objectForKey:kBRadius] floatValue];
		float movedis = ccpDistance(bulletsp, bulletpos);
		float transparency = (1.5-movedis/bulletrange)/1.5;
		if(sh>=3)
			drawBullet(bulletpos, bulletradius, bulletrota,CGRectMultiple(bulletcolor, transparency), BlackWithAlpha(0),sh);
		else if(sh==1){ 
			drawLaser(bulletsp, bulletpos, BlackWithAlpha(0),CGRectMultiple(bulletcolor, transparency), BlackWithAlpha(0), bulletradius/2);
			drawBullet(bulletpos, bulletradius*0.8, bulletrota,CGRectMultiple(bulletcolor, transparency), BlackWithAlpha(0),6);
		}
		else if(sh==2){
			drawLaser(bulletsp, bulletpos, BlackWithAlpha(0),CGRectMultiple(bulletcolor, transparency), BlackWithAlpha(0), bulletradius/2);
			drawBullet(bulletpos, bulletradius*0.8, bulletrota,CGRectMultiple(bulletcolor, transparency), BlackWithAlpha(0),3);
		}
	}
	for(ApostleBasis *e in gview.allEnemies){
		for(NSMutableDictionary *ebullet in e.bullets){
			int sh = [[ebullet objectForKey:kBShape] intValue];			
			CGPoint bulletsp = [[ebullet objectForKey:kBStart] CGPointValue];
			CGPoint bulletpos = [[ebullet objectForKey:kBPosition] CGPointValue];
			CGRect bulletcolor = [[ebullet objectForKey:kBColor] CGRectValue];
			float bulletrota = [[ebullet objectForKey:kBRotation] floatValue];
			float bulletrange = [[ebullet objectForKey:kBRange] floatValue];
			float bulletradius = [[ebullet objectForKey:kBRadius] floatValue];
			float movedis = ccpDistance(bulletsp, bulletpos);
			float transparency = (1.5-movedis/bulletrange)/1.5;
			if(sh>=3)
				drawBullet(bulletpos, bulletradius, bulletrota,CGRectMultiple(bulletcolor, transparency), BlackWithAlpha(0),sh);
			else if(sh==1){ 
				drawLaser(bulletsp, bulletpos, BlackWithAlpha(0),CGRectMultiple(bulletcolor, transparency), BlackWithAlpha(0), bulletradius/2);
				drawBullet(bulletpos, bulletradius*0.8, bulletrota,CGRectMultiple(bulletcolor, transparency), BlackWithAlpha(0),6);
			}
			else if(sh==2){
				drawLaser(bulletsp, bulletpos, BlackWithAlpha(0),CGRectMultiple(bulletcolor, transparency), BlackWithAlpha(0), bulletradius/2);
				drawBullet(bulletpos, bulletradius*0.8, bulletrota,CGRectMultiple(bulletcolor, transparency), BlackWithAlpha(0),3);
			}
		}
	}
	//draw explosion effect
	
	for(NSMutableDictionary *acounter in counterForExplosion){
		int t = [[acounter objectForKey:kTime] intValue];
		int mt = [[acounter objectForKey:kMaxTime] intValue];
		float s = [[acounter objectForKey:kWaveSpeed] floatValue];
		int sh = [[acounter objectForKey:kWaveShape] intValue];
		CGPoint ep = [[acounter objectForKey:kCenter] CGPointValue];
		CGRect color = [[acounter objectForKey:kColor] CGRectValue];
		CGRect newc = CGRectMultiple(color,(float)t/mt);
		if(sh<3)
			sh=64;
		drawExplosionWave(ep, (mt-t)*s, (mt-t)*s+28, BlackWithAlpha(0), newc, sh);
		drawExplosionWave(ep, (mt-t)*s+28, (mt-t)*s+35, newc, BlackWithAlpha(0), sh);
	}
	//show messages
	for(NSMutableDictionary *amessage in messagesToShow){
		int t = [[amessage objectForKey:kTime] intValue];
		int mt = [[amessage objectForKey:kMaxTime] intValue];
		CGPoint mepos = [[amessage objectForKey:kCenter] CGPointValue];
		CCLabelTTF *mes = [amessage objectForKey:kString];
		if(t==mt-1 && ![mes parent]){
			[self addChild:mes];
		}
		if(t==1){
			[mes removeFromParentAndCleanup:YES];
		}
		mes.position = mepos;
		mes.opacity = (float)t/mt*255.0;
	}
}

-(void)update:(ccTime)delta{
	NSMutableArray *allexplosions = [NSMutableArray arrayWithArray:counterForExplosion];
	int ec = 0;
	for(NSMutableDictionary *acounter in allexplosions){
		int time = [[acounter objectForKey:kTime] intValue];
		if(time>=0){
			time--;
			NSMutableDictionary *newcounter = [NSMutableDictionary dictionaryWithDictionary:acounter];
			[newcounter setObject:[NSNumber numberWithInt:time] forKey:kTime];
			[counterForExplosion replaceObjectAtIndex:ec withObject:newcounter];
			ec++;
	    }
		else 
			[counterForExplosion removeObjectAtIndex:ec];
	}
	
	NSMutableArray *allmessages = [NSMutableArray arrayWithArray:messagesToShow];
	int mc = 0;
	
	for(NSMutableDictionary *amessage in allmessages){
		int time = [[amessage objectForKey:kTime] intValue];
		if(time>=0){
			time--;
			NSMutableDictionary *newmessage = [NSMutableDictionary dictionaryWithDictionary:amessage];
			[newmessage setObject:[NSNumber numberWithInt:time] forKey:kTime];
			[messagesToShow replaceObjectAtIndex:mc withObject:newmessage];
			mc++;
	    }
		else 
			[messagesToShow removeObjectAtIndex:mc];
	}
}
-(void)dealloc{
	[super dealloc];
	//[counterForExplosion removeAllObjects];
	//[messagesToShow removeAllObjects];
	[counterForExplosion release];
	[messagesToShow release];
}
-(id)init{
	if((self=[super init])){
		instanceOfAML = self;
		counterForExplosion = [[NSMutableArray alloc] init];
		messagesToShow = [[NSMutableArray alloc] init];
			
		moveEffect = [[[CCParticleFireworks alloc] initWithTotalParticles:200] autorelease];
		moveEffect.startSize = 0.5f;
		moveEffect.startSizeVar = 0;
		moveEffect.life=0;
		ccColor4F black;
		black.r = black.g = black.b = black.a = 0.0f;
		moveEffect.startColorVar = moveEffect.endColorVar = moveEffect.startColor = black;
		[self addChild:moveEffect];
		[self scheduleUpdate];
	}
	return self;
}

-(void)showExplosionAt:(CGPoint)center WithTime:(float)sec Color:(CGRect)color WaveSpeed:(float)s{
	NSMutableDictionary *explosion = [NSDictionary dictionaryWithObjectsAndKeys:
									  [NSValue valueWithCGPoint:center],kCenter,
									  [NSValue valueWithCGRect:color],kColor,
									  [NSNumber numberWithInt:(int)(sec*30)],kTime,
									  [NSNumber numberWithInt:(int)(sec*30)],kMaxTime,
									  [NSNumber numberWithFloat:s],kWaveSpeed,
									  [NSNumber numberWithInt:0],kWaveShape,
									  nil];
	[counterForExplosion addObject:explosion];	
}
-(void)showExplosionAt:(CGPoint)center WithTime:(float)sec Color:(CGRect)color WaveSpeed:(float)s Shape:(int)sh{
	NSMutableDictionary *explosion = [NSDictionary dictionaryWithObjectsAndKeys:
									  [NSValue valueWithCGPoint:center],kCenter,
									  [NSValue valueWithCGRect:color],kColor,
									  [NSNumber numberWithInt:(int)(sec*30)],kTime,
									  [NSNumber numberWithInt:(int)(sec*30)],kMaxTime,
									  [NSNumber numberWithFloat:s],kWaveSpeed,
									  [NSNumber numberWithInt:sh],kWaveShape,
									  nil];
	[counterForExplosion addObject:explosion];	
}

-(void)addMessage:(NSString *)string At:(CGPoint)pos For:(float)sec FontSize:(int)s{
	NSMutableDictionary *amessage = [NSDictionary dictionaryWithObjectsAndKeys:
									[CCLabelTTF labelWithString:string fontName:@"Verdana" fontSize:s],kString,
									 [NSValue valueWithCGPoint:pos],kCenter,
									 [NSNumber numberWithInt:(int)(sec*30)],kMaxTime,
									 [NSNumber numberWithInt:(int)(sec*30)],kTime,nil];
	[messagesToShow addObject:amessage];	
}

-(void)showMoveEffectAt:(CGPoint)pos WithRotation:(float)rota Speed:(float)speed{
	//Set moveEffect
	moveEffect.position = pos;
	moveEffect.gravity = ccp(0,0);
	moveEffect.speed = speed*5+5;
	moveEffect.speedVar = 10;
	moveEffect.angle = 270 - rota;
	moveEffect.angleVar = 30;
	moveEffect.life = 1.0;
	moveEffect.lifeVar = 0;
	CGRect color = [GameView sharedGameView].mc.selfcolor;
	float scl = [GameView sharedGameView].scale;
	if(scl<1 && scl>=0.5)moveEffect.startSize = 0.8/scl;
	else if(scl<0.5)moveEffect.startSize = 1.6;
	else moveEffect.startSize = 0.6;
	ccColor4F sc;
	sc.r = color.origin.x;
	sc.g = color.origin.y;
	sc.b = color.size.width;
	sc.a = color.size.height;
	moveEffect.endColor = sc;
}
-(void)stopMoveEffect{
	moveEffect.life = 0;
}

@end
