/*
 * HAZ™ Dev Group: http://imhaz.com
 *
 * Copyright (c) 2007-2011 HAZ™ Inc.
 *
 * Written by Hercy Chang 2011-12-08
 *
 */

#include "CWColor.h"
#include "math.h"
CGRect ColorWithCode(int num,float a){
	CGRect c;
	switch (num) {
		case 0:
			c = YellowWithAlpha(a);
			break;
		case 1:
			c = PurpleWithAlpha(a);
			break;
		case 2:
			c = GreenWithAlpha(a);
			break;
		case 3:
			c = LightGreenWithAlpha(a);
			break;
		case 4:
			c = RedWithAlpha(a);
			break;
		case 5:
			c = BlueWithAlpha(a);
			break;
		default:
			c = BlackWithAlpha(a);
			break;
	}
	
	return c;
};
CGRect WhiteWithAlpha(float a){
	return CGRectMake(1, 1, 1, a);
};
CGRect BlackWithAlpha(float a){
	return CGRectMake(0, 0, 0, a);
};

CGRect RedWithAlpha(float a){
	return CGRectMake(1, 0, 0, a);
};
CGRect GreenWithAlpha(float a){
	return CGRectMake(0, 1, 0, a);
};
CGRect BlueWithAlpha(float a){
	return CGRectMake(0, 0, 1, a);
};
CGRect YellowWithAlpha(float a){
	return CGRectMake(1, 1, 0, a);
};
CGRect PurpleWithAlpha(float a){
	return CGRectMake(1, 0, 1, a);
};
CGRect LightGreenWithAlpha(float a){
	return CGRectMake(0, 1, 1, a);
};

//Mix: if factor==1,return new,when the factor grows, the result is nearer to the old
int CGRectFuzzyEqual(CGRect r1,CGRect r2,float delta){
	if(fabsf(r1.origin.x-r2.origin.x)<delta &&
	   fabsf(r1.origin.y-r2.origin.y)<delta &&
	   fabsf(r1.size.width-r2.size.width)<delta &&
	   fabsf(r1.size.height-r2.size.height)<delta )return 1;
	return 0;
		
}
float AngleOfTwoPoints(CGPoint center,CGPoint p){
	float distance = sqrt((p.x-center.x)*(p.x-center.x)+(p.y-center.y)*(p.y-center.y));
	if(distance==0)return 0;
	
	float angle;
	
	if(p.x>center.x)
		angle = acos((p.y-center.y)/distance)/2/3.1415926*360;
	else 
		angle = 360-acos((p.y-center.y)/distance)/2/3.1415926*360;
	
	return angle;
}
float nextAngle(float current,float target,float speed){
	float new;
	float clue = fabsf(current-target);
	if(clue<=speed || clue>=360){
		new = target;
	}
	else {
		new = current;
		if(clue<=180){
			if(target>current)
				new+=speed;
			else 
				new-=speed;
		}
		else {
			if(target>current)
				new-=speed;
			else 
				new+=speed;		
		}
	}
	return new;
}

float distanceWithCode(CGPoint c, CGPoint p, float r,int i){
	float result=0;
	float sr = r*sqrtf(3.0)/2;
	switch (i) {
		case 0:
			result = (p.x-c.x)/sr + 1;
			break;
		case 1:
			result = ((c.y-p.y)*sqrtf(3.0)+p.x-c.x)/2/sr + 1;
			break;
		case 2:
			result = ((c.y-p.y)*sqrtf(3.0)+c.x-p.x)/2/sr + 1;
			break;
		case 3:
			result = (c.x-p.x)/sr + 1;
			break;
		case 4:
			result = 1 - ((c.y-p.y)*sqrtf(3.0)+p.x-c.x)/2/sr;
			break;
		case 5:
			result = 1 - ((c.y-p.y)*sqrtf(3.0)+c.x-p.x)/2/sr;
			break;
		default:
			break;
	}
	return result;
}

CGPoint CalculateStarAndEnd(float s,float e){
	if(s<0 || s>360 || e<0 || e>360)
		return CGPointMake(-1, -1);
	
	float start,end;
	if(fabsf(s-e)<=180){
		if(s<=e){
			start = s;
			end = e;
		}
		else {
			start = e;
			end = s;
		}		
	}
	else {
		if(s<=e){
			start = e-360;
			end = s;
		}
		else {
			start = s-360;
			end = e;
		}
	}
	return CGPointMake(start, end);
}
CGPoint CenterCalculate(CGPoint c,float r,float x,float y){
	float sr = r*sqrtf(3.0)/2;
	CGPoint topleft = CGPointMake(c.x - sr, c.y + r);
	float dx = x*sr;
	float dy = (y/sqrtf(3.0)*2 + (1-x)/sqrtf(3.0)) * sr;
	CGPoint result = CGPointMake(topleft.x+dx, topleft.y-dy);
	return result;
}

int WhetherBeAttacked(CGPoint att,float attrota,CGPoint def,float adistance){
	float distance = sqrt((att.x-def.x)*(att.x-def.x)+(att.y-def.y)*(att.y-def.y));
	float angle = fabsf(AngleOfTwoPoints(att, def)-attrota);
	if(distance<=adistance && (angle<=30 || ((angle>=90)&&(angle<=150)) || ((angle>=210)&&(angle<=270)) || angle>=330))
		return 1;
	else return 0;
}

CGRect CGRectMultiple(CGRect oldrect,float factor){
	return CGRectMake(oldrect.origin.x*factor, oldrect.origin.y*factor, oldrect.size.width*factor, oldrect.size.height*factor);
}

CGRect CWColorTransform(CGRect start,CGRect end,float percent){
	CGRect new = start;
	new.origin.x+=(end.origin.x-start.origin.x)*percent;
	new.origin.y+=(end.origin.y-start.origin.y)*percent;
	new.size.width+=(end.size.width-start.size.width)*percent;
	new.size.height+=(end.size.height-start.size.height)*percent;
	return new;
}

CGRect CGRectMix(CGRect oldrect,CGRect newrect,float mixfactor){
	if(CGRectFuzzyEqual(oldrect, newrect, 0.1)==1)
	{
		return newrect;
	}
	float r = oldrect.origin.x+(newrect.origin.x-oldrect.origin.x)/mixfactor;
	float g = oldrect.origin.y+(newrect.origin.y-oldrect.origin.y)/mixfactor;
	float b = oldrect.size.width+(newrect.size.width-oldrect.size.width)/mixfactor;
	float a = oldrect.size.height+(newrect.size.height-oldrect.size.height)/mixfactor;
	return CGRectMake(r, g, b, a);
}

CGRect CGRectOfBar(CGPoint start, CGPoint end, float width){
	CGRect rect;
	rect = CGRectMake(start.x, start.y-width/2, end.x-start.x, width);
	return rect;
}
