//
//  DataLayer.m
//  ColorsWar
/*
 * HAZ™ Dev Group: http://imhaz.com
 *
 * Copyright (c) 2007-2011 HAZ™ Inc.
 *
 * Written by Hercy Chang 2011-12-08
 *
 */

#define kFighterSessionID @"StarWakeVSMode"
#define kMaxPacketSize 1024

#import "GameView.h"
#import "DataLayer.h"
#import "DataOfTheAncients.h"
#import "ApostleBasis.h"
#import "CWColor.h"

//
// for the sake of simplicity tank1 is the server and tank2 is the client
//
typedef enum {
	kServer,
	kClient
} gameNetwork;

@interface DataLayer (Private)

@end


@implementation DataLayer
@synthesize allMapPoints;
@synthesize mycolorfortest;
@synthesize gameMode;
@synthesize mapSize;
@synthesize length;
@synthesize kills,deathes;
@synthesize gameSession,gamePeerId,lastHeartbeatDate,connectionAlert,gameState;
@synthesize gameUniqueID;

static DataLayer* instanceOfDLayer;
+(DataLayer*) sharedDataLayer
{
	NSAssert(instanceOfDLayer != nil, @"DataLayer instance not yet initialized!");
	return instanceOfDLayer;
}
-(void)dealloc{
	[allMapPoints release];
	
	if(self.connectionAlert.visible) {
		[self.connectionAlert dismissWithClickedButtonIndex:-1 animated:NO];
	}
	self.connectionAlert = nil;
	
	[self invalidateSession:self.gameSession];
	self.gameSession = nil;
	self.gamePeerId = nil;
	
	[super dealloc];
}

-(id)init{
	if((self=[super init])){
		instanceOfDLayer = self;
		CGSize s = [[CCDirector sharedDirector] winSize];
		mapSize = CGSizeMake(s.width/10,s.height/10);
		length = HexShortRadius*10;
		allMapPoints = [[NSMutableArray alloc] init];
		kills = deathes = 0;
		
		gameUniqueID = 0;
		
		gameState = kStateStartGame;
		gamePacketNumber = 0;
		gameSession = nil;
		gamePeerId = nil;
		lastHeartbeatDate = nil;
	}
	return self;
}

-(void)addPointAt:(CGPoint)p Color:(CGRect)c BColor:(CGRect)bc Radius:(float)r Type:(int)t{
	NSMutableDictionary *apoint = [NSMutableDictionary dictionaryWithObjectsAndKeys:
								   [NSValue valueWithCGPoint:p],kMPosition,
								   [NSValue valueWithCGRect:c],kMColor,
								   [NSValue valueWithCGRect:bc],kMBackGround,
								   [NSNumber numberWithFloat:r],kMRadius,
								   [NSNumber numberWithFloat:0],kMPercent,
								   [NSNumber numberWithInt:t],kMType,
								   nil];
	[allMapPoints addObject:apoint];
}

-(BOOL)removePointAt:(int)index{
	int num = [allMapPoints count];
	
	if(index>num-1)
		return NO;
	else 
		[allMapPoints removeObjectAtIndex:index];
	return YES;
}

-(void)removeAllMapPoints{
	[allMapPoints removeAllObjects];
}

-(void)addKDToDOTA{
	[[DataOfTheAncients sharedDOTA] addK:kills De:deathes T:0];
	kills = 0;
	deathes = 0;
}

-(void)addMapPointsToDOTA{
	NSMutableArray *dp = [NSMutableArray arrayWithCapacity:6];
	
	for(NSMutableDictionary *amp in allMapPoints){
		CGPoint p = [[amp objectForKey:kMPosition] CGPointValue];
		NSValue *c = [amp objectForKey:kMColor];
		CGPoint converted = ccp(p.x/HexShortRadius,p.y/HexShortRadius);
		NSMutableDictionary *adp = [NSMutableDictionary dictionaryWithObjectsAndKeys:
									[NSValue valueWithCGPoint:converted],kMPosition,
									c,kMColor,nil];
		[dp addObject:adp];
	}
	[[DataOfTheAncients sharedDOTA] addDrawnPointsFrom:dp];
}

-(void)addMapPointsFromDOTA{
	[allMapPoints removeAllObjects];
	DataOfTheAncients *dota = [DataOfTheAncients sharedDOTA];
	[allMapPoints addObjectsFromArray:dota.drawnPoints];
}

-(NSString*)hintAtIndex:(int)i{
	switch (i%14) {
		case 0:
			return [NSString stringWithFormat:@"In the Awake mode the mission is to awake 6 stars"];
			break;
		case 1:
			return [NSString stringWithFormat:@"To open the map,tap the hex on topleft"];
			break;
		case 2:
			return [NSString stringWithFormat:@"The point shown in the map means that you should go there"];
			break;
		case 3:
			return [NSString stringWithFormat:@"To awake a star,open the green field first"];
			break;
		case 4:
			return [NSString stringWithFormat:@"Don't be hit with the green field opened"];
			break;
		case 5:
			return [NSString stringWithFormat:@"If a star awaked,the map will show you the next"];
			break;
		case 6:
			return [NSString stringWithFormat:@"Awake 6 stars to complete a mission"];
			break;
		case 7:
			return [NSString stringWithFormat:@"You may feel good if you build a cool mode"];
			break;
		case 8:
			return [NSString stringWithFormat:@"Shield makes the game easier"];
			break;
		case 9:
			return [NSString stringWithFormat:@"Releasing the red field causes huge damage in the center"];
			break;
		case 10:
			return [NSString stringWithFormat:@"Release the blink button to blink"];
			break;
		case 11:
			return [NSString stringWithFormat:@"Some enemies can jump to your back suddenly"];
			break;
		case 12:
			return [NSString stringWithFormat:@"This game has two modes:Awake and Survival"];
			break;
		case 13:
			return [NSString stringWithFormat:@"The only thing to do in the Survival mode is fight"];
			break;
		default:
			break;
	}
	return nil;
}

#pragma mark -
#pragma mark Peer Picker Related Methods

-(void)startPicker {
	GKPeerPickerController*		picker;
	gameUniqueID = (int)(CCRANDOM_0_1()*10000);
	self.gameState = kStatePicker;			// we're going to do Multiplayer!
	
	picker = [[GKPeerPickerController alloc] init]; // note: picker is released in various picker delegate methods when picker use is done.
	picker.delegate = self;
	[picker show]; // show the Peer Picker
}
#pragma mark GKPeerPickerControllerDelegate Methods

- (void)peerPickerControllerDidCancel:(GKPeerPickerController *)picker { 
	// Peer Picker automatically dismisses on user cancel. No need to programmatically dismiss.
    
	// autorelease the picker. 
	picker.delegate = nil;
    [picker autorelease]; 
	
	// invalidate and release game session if one is around.
	if(self.gameSession != nil)	{
		[self invalidateSession:self.gameSession];
		self.gameSession = nil;
	}
	
	// go back to start mode
	self.gameState = kStateStartGame;
} 

/*
 *	Note: No need to implement -peerPickerController:didSelectConnectionType: delegate method since this app does not support multiple connection types.
 *		- see reference documentation for this delegate method and the GKPeerPickerController's connectionTypesMask property.
 */

//
// Provide a custom session that has a custom session ID. This is also an opportunity to provide a session with a custom display name.
//
- (GKSession *)peerPickerController:(GKPeerPickerController *)picker sessionForConnectionType:(GKPeerPickerConnectionType)type { 
	GKSession *session = [[GKSession alloc] initWithSessionID:kFighterSessionID displayName:nil sessionMode:GKSessionModePeer]; 
	return [session autorelease]; // peer picker retains a reference, so autorelease ours so we don't leak.
}

- (void)peerPickerController:(GKPeerPickerController *)picker didConnectPeer:(NSString *)peerID toSession:(GKSession *)session { 
	// Remember the current peer.
	self.gamePeerId = peerID;  // copy
	
	// Make sure we have a reference to the game session and it is set up
	self.gameSession = session; // retain
	self.gameSession.delegate = self; 
	[self.gameSession setDataReceiveHandler:self withContext:NULL];
	
	// Done with the Peer Picker so dismiss it.
	[picker dismiss];
	picker.delegate = nil;
	[picker autorelease];
	
	// Start Multiplayer game by entering a cointoss state to determine who is server/client.
	self.gameState = kStateMultiplayer;
} 
#pragma mark -
#pragma mark Session Related Methods

//
// invalidate session
//
- (void)invalidateSession:(GKSession *)session {
	if(session != nil) {
		[session disconnectFromAllPeers]; 
		session.available = NO; 
		[session setDataReceiveHandler: nil withContext: NULL]; 
		session.delegate = nil; 
	}
}
#pragma mark Data Send/Receive Methods

/*
 * Getting a data packet. This is the data receive handler method expected by the GKSession. 
 * We set ourselves as the receive data handler in the -peerPickerController:didConnectPeer:toSession: method.
 */
- (void)receiveData:(NSData *)data fromPeer:(NSString *)peer inSession:(GKSession *)session context:(void *)context { 
	NSDictionary *dic = [NSKeyedUnarchiver unarchiveObjectWithData:data];
	int packetID = [[dic objectForKey:kDataType] integerValue];
	static double timedif = 0;
	switch( packetID ) {
		case NETWORK_COINTOSS:
		{
			int dt = [[dic objectForKey:@"dtimes"] intValue];
			self.kills = dt;
		}
			break;
		case NETWORK_INIT_DATA:
		{
			timedif = 0;
			NSDate *otherd = [dic objectForKey:sDKTime];
			timedif = [otherd timeIntervalSinceNow];
			
			GameView *gview = [GameView sharedGameView];
			ApostleBasis *e = [ApostleBasis createFrom:dic];
			NSMutableArray *es = gview.allEnemies;
			if([es count]==0){
				[gview addChild:e];
				[es addObject:e];
				[e becomeActiveVS];
			}
	    }
			break;
		case NETWORK_STREAM_DATA:
		{
			NSDate *sd = [dic objectForKey:sDKTime];
			double timepast = [sd timeIntervalSinceNow];
			
			if(fabs(timepast)>60+fabs(timedif))
				return;
			
			NSMutableArray *es = [GameView sharedGameView].allEnemies;

			if ([es count]==1) {
				ApostleBasis *e = [es objectAtIndex:0];
				[e refreshFrom:dic];
			}
		}
			break;
		case NETWORK_HEARTBEAT:
		{
		}
			break;
		default:
			// error
			break;
	}
}

- (void)sendNetworkPacket:(GKSession *)session packetID:(int)packetID withData:(void *)data ofLength:(int)l reliable:(BOOL)howtosend {
	if(howtosend == YES)
		[session sendData:data toPeers:[NSArray arrayWithObject:gamePeerId] withDataMode:GKSendDataReliable error:nil];
	else {
		[session sendData:data toPeers:[NSArray arrayWithObject:gamePeerId] withDataMode:GKSendDataUnreliable error:nil];
	}

}
#pragma mark GKSessionDelegate Methods

// we've gotten a state change in the session
- (void)session:(GKSession *)session peer:(NSString *)peerID didChangeState:(GKPeerConnectionState)state { 
	if(self.gameState == kStatePicker) {
		return;				// only do stuff if we're in multiplayer, otherwise it is probably for Picker
	}
	
	if(state == GKPeerStateDisconnected) {
		// We've been disconnected from the other peer.
		
		// Update user alert or throw alert if it isn't already up
		NSString *message = [NSString stringWithFormat:@"Could not reconnect with %@.", [session displayNameForPeer:peerID]];
		if((self.gameState == kStateMultiplayerReconnect) && self.connectionAlert && self.connectionAlert.visible) {
			self.connectionAlert.message = message;
		}
		else {
			UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"Lost Connection" message:message delegate:self cancelButtonTitle:@"End Game" otherButtonTitles:nil];
			self.connectionAlert = alert;
			[alert show];
			[alert release];
		}
		
		// go back to start mode
		self.gameState = kStateStartGame; 
	} 
} 
#pragma mark -
#pragma mark UIAlertViewDelegate Methods

// Called when an alert button is tapped.
- (void)alertView:(UIAlertView *)alertView clickedButtonAtIndex:(NSInteger)buttonIndex {
	// 0 index is "End Game" button
	if(buttonIndex == 0) {
		self.gameState = kStateStartGame; 
	}
}

//
// setter for gameState property
//
- (void)setGameState:(NSInteger)newState {
	if(newState == kStateStartGame) {
		if(self.gameSession) {
			// invalidate session and release it.
			[self invalidateSession:self.gameSession];
			self.gameSession = nil;
		}
		self.kills = 0;
		self.deathes = 0;
		// reset players to initial positions
	}
	
	gameState = newState;
}

@end
