/*
 * HAZ™ Dev Group: http://imhaz.com
 *
 * Copyright (c) 2007-2011 HAZ™ Inc.
 *
 * Written by Hercy Chang 2011-12-08
 *
 */

#import "GameMap.h"
#import "GameView.h"
#import "DataLayer.h"
#import "DataOfTheAncients.h"
#import "CWColor.h"

@implementation GameMap

void drawBoundary(CGSize msize,CGRect bcolor)
{	
	float delta = 35;

	GLfloat vertices[20] = {
		0,0,delta,delta,
		0,msize.height,delta,msize.height-delta,
		msize.width,msize.height,msize.width-delta,msize.height-delta,
		msize.width,0,msize.width-delta,delta,
	    0,0,delta,delta,
	};
	GLfloat colors[40];
	colors[0] = colors[8] = colors[16] = colors[24] = colors[32] = bcolor.origin.x;
	colors[1] = colors[9] = colors[17] = colors[25] = colors[33] = bcolor.origin.y;
	colors[2] = colors[10] = colors[18] = colors[26] = colors[34] = bcolor.size.width;
	colors[3] = colors[11] = colors[19] = colors[27] = colors[35] = bcolor.size.height;
	
	colors[4] = colors[12] = colors[20] = colors[28] = colors[36] = 0;
	colors[5] = colors[13] = colors[21] = colors[29] = colors[37] = 0;
	colors[6] = colors[14] = colors[22] = colors[30] = colors[38] = 0;
	colors[7] = colors[15] = colors[23] = colors[31] = colors[39] = 0;
	
	glDisable(GL_TEXTURE_2D);
	glDisableClientState(GL_TEXTURE_COORD_ARRAY);
	//glDisableClientState(GL_COLOR_ARRAY);
	
	glColorPointer(4, GL_FLOAT, 0, colors);
	glVertexPointer(2, GL_FLOAT, 0, vertices);	
	glDrawArrays(GL_TRIANGLE_STRIP, 0, 10);
	
	// restore default state
	glEnableClientState(GL_COLOR_ARRAY);
	glEnableClientState(GL_TEXTURE_COORD_ARRAY);
	glEnable(GL_TEXTURE_2D);	
	
}
void advancedHex( CGPoint center,float r1, float r2, CGRect color1, CGRect color2, int numofpoint)
{
	const float coef = (float)M_PI*2/numofpoint;
	
	GLfloat *vertices = calloc( sizeof(GLfloat)*2*(numofpoint+1)*2, 1);
	GLfloat *colors = calloc( sizeof(GLfloat)*4*(numofpoint+1)*2, 1);
	if( ! vertices || ! colors)
		return;
	
	vertices[0] = center.x;
	vertices[1] = center.y+r1;
	vertices[2] = center.x;
	vertices[3] = center.y+r2;
	colors[0] = color1.origin.x;
	colors[1] = color1.origin.y;
	colors[2] = color1.size.width;
	colors[3] = color1.size.height;
	colors[4] = color2.origin.x;
	colors[5] = color2.origin.y;
	colors[6] = color2.size.width;
	colors[7] = color2.size.height;
	
	for(NSUInteger i=1;i<(numofpoint+1);i++)
	{
		float rads = i*coef;
		GLfloat ix1 = r1 * sinf(rads) + center.x;
		GLfloat iy1 = r1 * cosf(rads) + center.y;
		GLfloat ix2 = r2 * sinf(rads) + center.x;
		GLfloat iy2 = r2 * cosf(rads) + center.y;
		
		vertices[i*4] = ix1 * CC_CONTENT_SCALE_FACTOR();
		vertices[i*4+1] = iy1 * CC_CONTENT_SCALE_FACTOR();
		vertices[i*4+2] = ix2 * CC_CONTENT_SCALE_FACTOR();
		vertices[i*4+3] = iy2 * CC_CONTENT_SCALE_FACTOR();
		
		colors[i*8] = color1.origin.x;
		colors[i*8+1] = color1.origin.y;
		colors[i*8+2] = color1.size.width;
		colors[i*8+3] = color1.size.height;
		
		colors[i*8+4] = color2.origin.x;
		colors[i*8+5] = color2.origin.y;
		colors[i*8+6] = color2.size.width;
		colors[i*8+7] = color2.size.height;
	}
	
	// Default GL states: GL_TEXTURE_2D, GL_VERTEX_ARRAY, GL_COLOR_ARRAY, GL_TEXTURE_COORD_ARRAY
	// Needed states: GL_VERTEX_ARRAY, 
	// Unneeded states: GL_TEXTURE_2D, GL_TEXTURE_COORD_ARRAY, GL_COLOR_ARRAY	
	glDisable(GL_TEXTURE_2D);
	glDisableClientState(GL_TEXTURE_COORD_ARRAY);
	//glDisableClientState(GL_COLOR_ARRAY);
	
	glColorPointer(4, GL_FLOAT, 0, colors);
	glVertexPointer(2, GL_FLOAT, 0, vertices);	
	glDrawArrays(GL_TRIANGLE_STRIP, 0, (numofpoint+1)*2);
	
	// restore default state
	glEnableClientState(GL_COLOR_ARRAY);
	glEnableClientState(GL_TEXTURE_COORD_ARRAY);
	glEnable(GL_TEXTURE_2D);	
	
	free( vertices );
	free( colors );
	
}

void hexCircle( CGPoint center,float r1, float r2, CGRect sc1,CGRect ec1 ,CGRect sc2,CGRect ec2, int numofpoint,float radstart,float radend)
{
	if(r1==r2)return;
	
	float coef = (radend-radstart)/numofpoint;
	float pif = (float)M_PI/180;
	
	GLfloat *vertices = calloc( sizeof(GLfloat)*2*(numofpoint+1)*2, 1);
	GLfloat *colors = calloc( sizeof(GLfloat)*4*(numofpoint+1)*2, 1);
	if( ! vertices || ! colors)
		return;
	
	for(NSUInteger i=0;i<(numofpoint+1);i++)
	{
		float rads = (i*coef + radstart)*pif;
		GLfloat ix1 = r1 * sinf(rads) + center.x;
		GLfloat iy1 = r1 * cosf(rads) + center.y;
		GLfloat ix2 = r2 * sinf(rads) + center.x;
		GLfloat iy2 = r2 * cosf(rads) + center.y;
		
		vertices[i*4] = ix1 * CC_CONTENT_SCALE_FACTOR();
		vertices[i*4+1] = iy1 * CC_CONTENT_SCALE_FACTOR();
		vertices[i*4+2] = ix2 * CC_CONTENT_SCALE_FACTOR();
		vertices[i*4+3] = iy2 * CC_CONTENT_SCALE_FACTOR();
		
		float trf = sinf((float)i/numofpoint*3.1415926);
		CGRect color1 = CWColorTransform(sc1, ec1, trf);
		CGRect color2 = CWColorTransform(sc2, ec2, trf);
		
		colors[i*8] = color1.origin.x;
		colors[i*8+1] = color1.origin.y;
		colors[i*8+2] = color1.size.width;
		colors[i*8+3] = color1.size.height;
		
		colors[i*8+4] = color2.origin.x;
		colors[i*8+5] = color2.origin.y;
		colors[i*8+6] = color2.size.width;
		colors[i*8+7] = color2.size.height;
	}
	
	// Default GL states: GL_TEXTURE_2D, GL_VERTEX_ARRAY, GL_COLOR_ARRAY, GL_TEXTURE_COORD_ARRAY
	// Needed states: GL_VERTEX_ARRAY, 
	// Unneeded states: GL_TEXTURE_2D, GL_TEXTURE_COORD_ARRAY, GL_COLOR_ARRAY	
	glDisable(GL_TEXTURE_2D);
	glDisableClientState(GL_TEXTURE_COORD_ARRAY);
	//glDisableClientState(GL_COLOR_ARRAY);
	
	glColorPointer(4, GL_FLOAT, 0, colors);
	glVertexPointer(2, GL_FLOAT, 0, vertices);	
	glDrawArrays(GL_TRIANGLE_STRIP, 0, (numofpoint+1)*2);
	
	// restore default state
	glEnableClientState(GL_COLOR_ARRAY);
	glEnableClientState(GL_TEXTURE_COORD_ARRAY);
	glEnable(GL_TEXTURE_2D);	
	
	free( vertices );
	free( colors );
	
}

void linePlus(CGPoint sp,CGPoint ep,CGRect startc,CGRect endc,CGRect edgecolor,float width)
{
	float anglestoe = AngleOfTwoPoints(sp, ep);
	float radfactor = (float)M_PI/180;
	
	GLfloat vertices[12];
	GLfloat colors[24];
	
	float dx1 = width/2*sinf((anglestoe+90)*radfactor);
	float dy1 = width/2*cosf((anglestoe+90)*radfactor);
	float dx2 = width/2*sinf((anglestoe+270)*radfactor);
	float dy2 = width/2*cosf((anglestoe+270)*radfactor);
	
	vertices[0] = sp.x+dx1;
	vertices[1] = sp.y+dy1;
	vertices[2] = ep.x+dx1;
	vertices[3] = ep.y+dy1;
	vertices[4] = sp.x;
	vertices[5] = sp.y;
	vertices[6] = ep.x;
	vertices[7] = ep.y;
	vertices[8] = sp.x+dx2;
	vertices[9] = sp.y+dy2;
	vertices[10] = ep.x+dx2;
	vertices[11] = ep.y+dy2;
	
	colors[0] = colors[4] = colors[16] = colors[20] = edgecolor.origin.x;
	colors[1] = colors[5] = colors[17] = colors[21] = edgecolor.origin.y;
	colors[2] = colors[6] = colors[18] = colors[22] = edgecolor.size.width;
	colors[3] = colors[7] = colors[19] = colors[23] = edgecolor.size.height;
	
	colors[8] = startc.origin.x;
	colors[9] = startc.origin.y;
	colors[10] = startc.size.width;
	colors[11] = startc.size.height;
	
	colors[12] = endc.origin.x;
	colors[13] = endc.origin.y;
	colors[14] = endc.size.width;
	colors[15] = endc.size.height;
	
	glDisable(GL_TEXTURE_2D);
	glDisableClientState(GL_TEXTURE_COORD_ARRAY);
	
	glColorPointer(4, GL_FLOAT, 0, colors);
	glVertexPointer(2, GL_FLOAT, 0, vertices);	
	glDrawArrays(GL_TRIANGLE_STRIP, 0, 6);
	
	// restore default state
	glEnableClientState(GL_TEXTURE_COORD_ARRAY);
	glEnable(GL_TEXTURE_2D);		
	
}
-(void)draw{
	DataLayer *dl = [DataLayer sharedDataLayer];
	GameView *gl = [GameView sharedGameView];
	
	if(dl.gameMode!=GameModeFind)
		drawBoundary(gl.mapSize, YellowWithAlpha(0));

	if(dl.gameMode==GameModeExplore){
		
		for(NSMutableDictionary *apoint in dl.allMapPoints){
			CGPoint p = [[apoint objectForKey:kMPosition] CGPointValue];
			CGRect c = [[apoint objectForKey:kMColor] CGRectValue];
			CGRect bc = [[apoint objectForKey:kMBackGround] CGRectValue];
			float r = [[apoint objectForKey:kMRadius] floatValue];
			float per = [[apoint objectForKey:kMPercent] floatValue];
			int t = [[apoint objectForKey:kMType] intValue];
			
			CGRect thecolor = CGRectMultiple(c, per);
			float delta = r*0.03;
			
			if(delta<3)
				delta=3;
			advancedHex(p, 0,r-delta, thecolor, bc, t);
			advancedHex(p, r-delta,r, bc,BlackWithAlpha(0), t);
			
			hexCircle(p, r-delta*2, r, BlackWithAlpha(0), BlackWithAlpha(0), thecolor, thecolor, 64, 0, per*360);
			hexCircle(p, r, r+delta*2, thecolor, thecolor, BlackWithAlpha(0), BlackWithAlpha(0), 64, 0, per*360);
		}		
	}
	if(dl.gameMode==GameModeFind){
		acounter++;
		if(acounter>=18000)
			acounter=0;
		
		int mapnum = 0;
		int posnum = 0;
		for(NSMutableArray *amap in dl.allMapPoints){
			posnum = 0;
			for(NSMutableDictionary *apoint in amap){
				int tnum = mapnum*6 + posnum;
				float cf = (acounter-tnum*30)/100.0;
				if(cf<0)
					cf=0;
				if(cf>1)
					cf=1;
				CGPoint p = [[apoint objectForKey:kMPosition] CGPointValue];
				CGRect c = [[apoint objectForKey:kMColor] CGRectValue];
				
				float xadd;
				if((mapnum/10)%2==0)
					xadd = mapnum%10*screenSize.width;
				else 
					xadd = (9-mapnum%10)*screenSize.width;
				
				float yadd = mapnum/10*screenSize.height;
				CGPoint hexpos = ccpAdd(ccp(xadd,yadd), p);
				CGRect hexc = CGRectMultiple(c, cf);
				
				if(tnum%2==0){
					p1 = hexpos;
					c1 = hexc;
				}else {
					p2 = hexpos;
					c2 = hexc;
				}
				
				advancedHex(hexpos, 0, 10.0/gl.scale, hexc, BlackWithAlpha(0), 64);
				if(tnum)
					linePlus(p1, p2, c1, c2, BlackWithAlpha(0), 8.0/gl.scale);
				posnum++;
			}
			mapnum++;
		}		
	}
}
		   
-(void)dealloc{
	[super dealloc];
}
-(id)init{
	if((self=[super init])){
		screenSize = [[CCDirector sharedDirector] winSize];
		acounter = ([[DataOfTheAncients sharedDOTA].drawnPoints count]*6-18)*30;
		if(acounter<0)
			acounter=0;
		}
	return self;
}

@end
