//
//  MoveAndScale.m
//  ColorsWar
//
/*
 * HAZ™ Dev Group: http://imhaz.com
 *
 * Copyright (c) 2007-2011 HAZ™ Inc.
 *
 * Written by Hercy Chang 2011-12-08
 *
 */

#import "MoveAndScale.h"
#import "CWColor.h"

@implementation MoveAndScale

@synthesize colortodraw;
@synthesize angle,speed;
@synthesize isMoving;
@synthesize maxZoomfactor;
@synthesize zoomfactor;
@synthesize isScaling;
@synthesize shoot;

-(void)dealloc{
	[super dealloc];
}
-(id)init{
	if((self=[super init])){
		screenSize = [[CCDirector sharedDirector] winSize];
		colortodraw = WhiteWithAlpha(0.5);
		touchRect = CGRectMake(0, screenSize.height/8, screenSize.width, screenSize.height*31/40);
		radius = touchRect.size.height/4;
		self.isTouchEnabled = YES;
		isMoving = NO;
		isScaling = NO;
		shoot = NO;
		angle = speed = 0;
		zoomfactor = prezoomf = 1.0f;
		maxZoomfactor = 3.0;
		touchPoint = currentCenter = ccp(screenSize.width/2,screenSize.height/2);
	}
	return self;
}
void drawController( CGPoint center,float r1, float r2, CGRect color1, CGRect color2, int numofpoint)
{
	const float coef = (float)M_PI*2/numofpoint;
	
	GLfloat *vertices = calloc( sizeof(GLfloat)*2*(numofpoint+1)*2, 1);
	GLfloat *colors = calloc( sizeof(GLfloat)*4*(numofpoint+1)*2, 1);
	if( ! vertices || ! colors)
		return;
	
	vertices[0] = center.x;
	vertices[1] = center.y+r1;
	vertices[2] = center.x;
	vertices[3] = center.y+r2;
	colors[0] = color1.origin.x;
	colors[1] = color1.origin.y;
	colors[2] = color1.size.width;
	colors[3] = color1.size.height;
	colors[4] = color2.origin.x;
	colors[5] = color2.origin.y;
	colors[6] = color2.size.width;
	colors[7] = color2.size.height;
	
	for(NSUInteger i=1;i<(numofpoint+1);i++)
	{
		float rads = i*coef;
		GLfloat ix1 = r1 * sinf(rads) + center.x;
		GLfloat iy1 = r1 * cosf(rads) + center.y;
		GLfloat ix2 = r2 * sinf(rads) + center.x;
		GLfloat iy2 = r2 * cosf(rads) + center.y;
		
		vertices[i*4] = ix1 * CC_CONTENT_SCALE_FACTOR();
		vertices[i*4+1] = iy1 * CC_CONTENT_SCALE_FACTOR();
		vertices[i*4+2] = ix2 * CC_CONTENT_SCALE_FACTOR();
		vertices[i*4+3] = iy2 * CC_CONTENT_SCALE_FACTOR();
		
		colors[i*8] = color1.origin.x;
		colors[i*8+1] = color1.origin.y;
		colors[i*8+2] = color1.size.width;
		colors[i*8+3] = color1.size.height;
		
		colors[i*8+4] = color2.origin.x;
		colors[i*8+5] = color2.origin.y;
		colors[i*8+6] = color2.size.width;
		colors[i*8+7] = color2.size.height;
	}
	
	// Default GL states: GL_TEXTURE_2D, GL_VERTEX_ARRAY, GL_COLOR_ARRAY, GL_TEXTURE_COORD_ARRAY
	// Needed states: GL_VERTEX_ARRAY, 
	// Unneeded states: GL_TEXTURE_2D, GL_TEXTURE_COORD_ARRAY, GL_COLOR_ARRAY	
	glDisable(GL_TEXTURE_2D);
	glDisableClientState(GL_TEXTURE_COORD_ARRAY);
	//glDisableClientState(GL_COLOR_ARRAY);
	
	glColorPointer(4, GL_FLOAT, 0, colors);
	glVertexPointer(2, GL_FLOAT, 0, vertices);	
	glDrawArrays(GL_TRIANGLE_STRIP, 0, (numofpoint+1)*2);
	
	// restore default state
	glEnableClientState(GL_COLOR_ARRAY);
	glEnableClientState(GL_TEXTURE_COORD_ARRAY);
	glEnable(GL_TEXTURE_2D);	
	
	free( vertices );
	free( colors );
	
}
void drawTouchPoint( CGPoint center,float r1, float r2, CGRect color1, CGRect color2, int numofpoint,float rota)
{
	const float coef = (float)M_PI*2/numofpoint;
	
	GLfloat *vertices = calloc( sizeof(GLfloat)*2*(numofpoint+1)*2, 1);
	GLfloat *colors = calloc( sizeof(GLfloat)*4*(numofpoint+1)*2, 1);
	if( ! vertices || ! colors)
		return;
	
	for(NSUInteger i=0;i<(numofpoint+1);i++)
	{
		float rads = i*coef + rota/180*3.1415926;
		GLfloat ix1 = r1 * sinf(rads) + center.x;
		GLfloat iy1 = r1 * cosf(rads) + center.y;
		GLfloat ix2 = r2 * sinf(rads) + center.x;
		GLfloat iy2 = r2 * cosf(rads) + center.y;
		
		vertices[i*4] = ix1 * CC_CONTENT_SCALE_FACTOR();
		vertices[i*4+1] = iy1 * CC_CONTENT_SCALE_FACTOR();
		vertices[i*4+2] = ix2 * CC_CONTENT_SCALE_FACTOR();
		vertices[i*4+3] = iy2 * CC_CONTENT_SCALE_FACTOR();
		
		colors[i*8] = color1.origin.x;
		colors[i*8+1] = color1.origin.y;
		colors[i*8+2] = color1.size.width;
		colors[i*8+3] = color1.size.height;
		
		colors[i*8+4] = color2.origin.x;
		colors[i*8+5] = color2.origin.y;
		colors[i*8+6] = color2.size.width;
		colors[i*8+7] = color2.size.height;
	}
	
	// Default GL states: GL_TEXTURE_2D, GL_VERTEX_ARRAY, GL_COLOR_ARRAY, GL_TEXTURE_COORD_ARRAY
	// Needed states: GL_VERTEX_ARRAY, 
	// Unneeded states: GL_TEXTURE_2D, GL_TEXTURE_COORD_ARRAY, GL_COLOR_ARRAY	
	glDisable(GL_TEXTURE_2D);
	glDisableClientState(GL_TEXTURE_COORD_ARRAY);
	//glDisableClientState(GL_COLOR_ARRAY);
	
	glColorPointer(4, GL_FLOAT, 0, colors);
	glVertexPointer(2, GL_FLOAT, 0, vertices);	
	glDrawArrays(GL_TRIANGLE_STRIP, 0, (numofpoint+1)*2);
	
	// restore default state
	glEnableClientState(GL_COLOR_ARRAY);
	glEnableClientState(GL_TEXTURE_COORD_ARRAY);
	glEnable(GL_TEXTURE_2D);	
	
	free( vertices );
	free( colors );
	
}

-(void)draw{	
	if([CCDirector sharedDirector].isPaused)
		return;
	if(!isMoving)return;
	drawController(currentCenter, radius, radius*1.45, BlackWithAlpha(0), colortodraw, 100);
	drawController(currentCenter, radius*1.45, radius*1.5, colortodraw, BlackWithAlpha(0), 100);
	drawTouchPoint(touchPoint,0,radius/4,colortodraw,BlackWithAlpha(0),3,angle);
}

-(void)ccTouchesBegan:(NSSet*)touches withEvent:(UIEvent*)event
{
	NSSet *alltouch = [event allTouches];
	//tapcount==1,turn and move
	if([alltouch count]==1){
		
		isScaling=NO;
		prezoomf = zoomfactor;

		UITouch *touch = [alltouch anyObject];
		CGPoint location = [touch locationInView:[touch view]];
		CGPoint convertedLocation = [[CCDirector sharedDirector] convertToGL:location];
		
		if(!CGRectContainsPoint(touchRect, convertedLocation) ){
			speed=0;
			isMoving=NO;
			return;
		}
		float distance = ccpDistance(convertedLocation, currentCenter);
		angle = AngleOfTwoPoints(currentCenter, convertedLocation);
		isMoving=YES;
		
		if(distance<radius*1.5){
			speed = (distance/radius+1)*(distance/radius+1)*3.5;
			touchPoint = convertedLocation;
		}
		else{
			speed = 20;
			touchPoint = ccpAdd(currentCenter, ccp(radius*sinf(angle/180*3.1415926),radius*cosf(angle/180*3.1415926)));
		}
				
	}
	//tapcount==2:scale
	else if([alltouch count]==2){
		
		self.visible = YES;
		UITouch *touch1 = [[alltouch allObjects] objectAtIndex:0];
		UITouch *touch2 = [[alltouch allObjects] objectAtIndex:1];
		CGPoint location1 = [touch1 locationInView:[touch1 view]];
		CGPoint location2 = [touch2 locationInView:[touch2 view]];
		CGPoint p1 = [[CCDirector sharedDirector] convertToGL:location1];
		CGPoint p2 = [[CCDirector sharedDirector] convertToGL:location2];

		if(!CGRectContainsPoint(touchRect, p1) || !CGRectContainsPoint(touchRect, p2) ) {
			isScaling=NO;
			return;
		}
		isMoving = NO;
		startdistance = ccpDistance(p1, p2);
		
	}
	// tapcount==3
	else if([alltouch count]==3){
		shoot=YES;
	}
}
-(void)ccTouchesMoved:(NSSet*)touches withEvent:(UIEvent*)event
{
	NSSet *alltouch = [event allTouches];
	if([alltouch count]==1){
		
		isScaling=NO;
		prezoomf = zoomfactor;

		UITouch *touch = [alltouch anyObject];
		CGPoint location = [touch locationInView:[touch view]];
		CGPoint convertedLocation = [[CCDirector sharedDirector] convertToGL:location];
		
		if(!CGRectContainsPoint(touchRect, convertedLocation) ){
			speed=0;
			isMoving=NO;
			return;
		}
		touchPoint = convertedLocation;

		float distance = ccpDistance(convertedLocation, currentCenter);
		angle = AngleOfTwoPoints(currentCenter, convertedLocation);
		isMoving=YES;
		
		if(distance<radius*1.5){
			speed = distance/radius*distance/radius*10;
			touchPoint = convertedLocation;
		}
		else{
			speed = 20;
			touchPoint = ccpAdd(currentCenter, ccp(radius*sinf(angle/180*3.1415926),radius*cosf(angle/180*3.1415926)));
		}
		
	}
	else if([alltouch count]==2){
		
		self.visible = YES;
		UITouch *touch1 = [[alltouch allObjects] objectAtIndex:0];
		UITouch *touch2 = [[alltouch allObjects] objectAtIndex:1];
		CGPoint location1 = [touch1 locationInView:[touch1 view]];
		CGPoint location2 = [touch2 locationInView:[touch2 view]];
		CGPoint p1 = [[CCDirector sharedDirector] convertToGL:location1];
		CGPoint p2 = [[CCDirector sharedDirector] convertToGL:location2];
		
		if(!CGRectContainsPoint(touchRect, p1) || !CGRectContainsPoint(touchRect, p2) ) {
			isScaling=NO;
			return;
		}
		isMoving = NO;
		
		float zoomf = ccpDistance(p1, p2)/startdistance;
		if(zoomf<0.9){
			zoomfactor = prezoomf*(zoomf+0.05);
		}
		if(zoomf>1.1){
			zoomfactor = prezoomf*(zoomf-0.05);
		}
		
		if(zoomfactor>screenSize.width/200)zoomfactor=screenSize.width/200;
		//else if(zoomfactor<screenSize.width/1096)zoomfactor=screenSize.width/1096;
		else if(zoomfactor<1/maxZoomfactor)zoomfactor = 1/maxZoomfactor;
		isScaling=YES;
		
	}
	else if([alltouch count]==3){
		UITouch *touch1 = [[alltouch allObjects] objectAtIndex:0];
		UITouch *touch2 = [[alltouch allObjects] objectAtIndex:1];
		UITouch *touch3 = [[alltouch allObjects] objectAtIndex:2];
		CGPoint location1 = [touch1 locationInView:[touch1 view]];
		CGPoint location2 = [touch2 locationInView:[touch2 view]];
		CGPoint location3 = [touch2 locationInView:[touch3 view]];
		CGPoint p1 = [[CCDirector sharedDirector] convertToGL:location1];
		CGPoint p2 = [[CCDirector sharedDirector] convertToGL:location2];
		CGPoint p3 = [[CCDirector sharedDirector] convertToGL:location3];
		
		if(!CGRectContainsPoint(touchRect, p1) || !CGRectContainsPoint(touchRect, p2) || !CGRectContainsPoint(touchRect, p3)) {
			return;
		}
		
		isMoving = YES;
		speed = 0;
		float xpos = (p1.x+p2.x+p3.x)/3;
		if(xpos>=radius*1.5 && xpos<=(screenSize.width - radius*1.5))
			currentCenter.x = xpos;
	}
	
}
-(void)ccTouchesEnded:(NSSet*)touches withEvent:(UIEvent*)event{
	isScaling=NO;
	shoot=NO;	
	prezoomf = zoomfactor;

	UITouch *touch = [[touches allObjects] lastObject];
	CGPoint location = [touch locationInView:[touch view]];
	CGPoint convertedLocation = [[CCDirector sharedDirector] convertToGL:location];
	if(CGRectContainsPoint(touchRect, convertedLocation)){
		speed = 0;
		isMoving=NO;
	}
}
-(void)ccTouchesCancelled:(NSSet*)touches withEvent:(UIEvent*)event{
	isScaling=NO;
	shoot=NO;	
	prezoomf = zoomfactor;
	speed = 0;
	isMoving=NO;
	shoot=NO;
}

@end
