/*
 * HAZ™ Dev Group: http://imhaz.com
 *
 * Copyright (c) 2007-2011 HAZ™ Inc.
 *
 *
 */

#import <Foundation/Foundation.h>

@class CCAnimation;

/** Singleton that manages the Animations.
 It saves in a cache the animations. You should use this class if you want to save your animations in a cache.

 Before v0.99.5, the recommend way was to save them on the CCSprite. Since v0.99.5, you should use this class instead.
 
 @since v0.99.5
 */
@interface CCAnimationCache : NSObject
{
	NSMutableDictionary *animations_;
}

/** Retruns ths shared instance of the Animation cache */
+ (CCAnimationCache *) sharedAnimationCache;

/** Purges the cache. It releases all the CCAnimation objects and the shared instance.
 */
+(void)purgeSharedAnimationCache;

/** Adds a CCAnimation with a name.
 */
-(void) addAnimation:(CCAnimation*)animation name:(NSString*)name;

/** Deletes a CCAnimation from the cache.
 */
-(void) removeAnimationByName:(NSString*)name;

/** Returns a CCAnimation that was previously added.
 If the name is not found it will return nil.
 You should retain the returned copy if you are going to use it.
 */
-(CCAnimation*) animationByName:(NSString*)name;

@end
